=begin
Each possible object in the resulting XML tree must be of a class that includes
this module; only then can the XmlParser build the tree.

Note that you have to create a class method xmlName() as well (but a Ruby
module can not enforce this; neither can a Java interface, btw). If you forget
this, XmlParser.new() will generate a RunTimeError.

For convenience, a default printing routine is given in this module. It is
/strongly/ adviced you overload childrenStr() in any XmlElt that you use that
can have children
=end
module XmlElt
  INDENTSTR = "\t"

  def addAttribute(attrib, value=nil)
    @attrib.store(attrib, value)
  end
  def addContent(content)
    @content << content
  end
  # when adding the child, you can set the parent of the child, if you wish
  def addChild(child) end

  def indentStr(level)
    result = ""
    level.times { result += INDENTSTR }
    result
  end

  def attribStr()
    result = ""
    @attrib.each_pair do |attrib, value|
      result += " "+attrib
      result += '="'+value+'"'  if value
    end
    result
  end

  def openingTagStr(level)
    indentStr(level)+"<"+type().xmlName()+attribStr()+">"
  end

  def contentStr(level)
    result = indentStr(level)
    @content.each do |content|
      result += content.to_s(level+1)
    end
    result
  end

  def childrenStr(level)
    # empty string, since we know nothing about children
    ""
  end

  def closingTagStr(level)
    result = indentStr(level)
    result += "</"+type().xmlName()+">"
  end

  def to_s(level=0)
    result = openingTagStr(level)
    content = contentStr(level+1)
    content += childrenStr(level+1)
    if (content.strip() != "")
      result += "\n"+content+closingTagStr(level)
    else
      result[-1, 0] = "/"
    end
    result += "\n"
  end
end

class AnonXml
  include XmlElt
  def initialize(name)
    @name = name
    @attrib = {}
    @content = []
  end
end
